@@ -1,11 +1,15 @@
 import boto
 from boto.dynamodb2 import exceptions
 from boto.dynamodb2.fields import (HashKey, RangeKey,
-                                   AllIndex, KeysOnlyIndex, IncludeIndex)
+                                   AllIndex, KeysOnlyIndex, IncludeIndex,
+                                   GlobalAllIndex, GlobalKeysOnlyIndex,
+                                   GlobalIncludeIndex)
 from boto.dynamodb2.items import Item
 from boto.dynamodb2.layer1 import DynamoDBConnection
 from boto.dynamodb2.results import ResultSet, BatchGetResultSet
-from boto.dynamodb2.types import Dynamizer, FILTER_OPERATORS, QUERY_OPERATORS
+from boto.dynamodb2.types import (Dynamizer, FILTER_OPERATORS, QUERY_OPERATORS,
+                                  STRING)
+from boto.exception import JSONResponseError
 
 
 class Table(object):
@@ -21,7 +25,7 @@ class Table(object):
     max_batch_get = 100
 
     def __init__(self, table_name, schema=None, throughput=None, indexes=None,
-                 connection=None):
+                 global_indexes=None, connection=None):
         """
         Sets up a new in-memory ``Table``.
 
@@ -48,6 +52,10 @@ class Table(object):
         Optionally accepts a ``indexes`` parameter, which should be a list of
         ``BaseIndexField`` subclasses representing the desired indexes.
 
+        Optionally accepts a ``global_indexes`` parameter, which should be a
+        list of ``GlobalBaseIndexField`` subclasses representing the desired
+        indexes.
+
         Optionally accepts a ``connection`` parameter, which should be a
         ``DynamoDBConnection`` instance (or subclass). This is primarily useful
         for specifying alternate connection parameters.
@@ -67,13 +75,22 @@ class Table(object):
             ...     'write': 10,
             ... }, indexes=[
             ...     KeysOnlyIndex('MostRecentlyJoined', parts=[
+            ...         HashKey('username')
             ...         RangeKey('date_joined')
             ...     ]),
-            ... ],
-            ... connection=dynamodb2.connect_to_region('us-west-2',
-		    ...     aws_access_key_id='key',
-		    ...     aws_secret_access_key='key',
-	        ... ))
+            ... ], global_indexes=[
+            ...     GlobalAllIndex('UsersByZipcode', parts=[
+            ...         HashKey('zipcode'),
+            ...         RangeKey('username'),
+            ...     ],
+            ...     throughput={
+            ...       'read':10,
+            ...       'write":10,
+            ...     }),
+            ... ], connection=dynamodb2.connect_to_region('us-west-2',
+            ...     aws_access_key_id='key',
+            ...     aws_secret_access_key='key',
+            ... ))
 
         """
         self.table_name = table_name
@@ -84,6 +101,7 @@ class Table(object):
         }
         self.schema = schema
         self.indexes = indexes
+        self.global_indexes = global_indexes
 
         if self.connection is None:
             self.connection = DynamoDBConnection()
@@ -95,7 +113,7 @@ class Table(object):
 
     @classmethod
     def create(cls, table_name, schema, throughput=None, indexes=None,
-               connection=None):
+               global_indexes=None, connection=None):
         """
         Creates a new table in DynamoDB & returns an in-memory ``Table`` object.
 
@@ -127,6 +145,10 @@ class Table(object):
         Optionally accepts a ``indexes`` parameter, which should be a list of
         ``BaseIndexField`` subclasses representing the desired indexes.
 
+        Optionally accepts a ``global_indexes`` parameter, which should be a
+        list of ``GlobalBaseIndexField`` subclasses representing the desired
+        indexes.
+
         Optionally accepts a ``connection`` parameter, which should be a
         ``DynamoDBConnection`` instance (or subclass). This is primarily useful
         for specifying alternate connection parameters.
@@ -142,7 +164,15 @@ class Table(object):
             ... }, indexes=[
             ...     KeysOnlyIndex('MostRecentlyJoined', parts=[
             ...         RangeKey('date_joined')
-            ...     ]),
+            ... ]), global_indexes=[
+            ...     GlobalAllIndex('UsersByZipcode', parts=[
+            ...         HashKey('zipcode'),
+            ...         RangeKey('username'),
+            ...     ],
+            ...     throughput={
+            ...       'read':10,
+            ...       'write':10,
+            ...     }),
             ... ])
 
         """
@@ -155,13 +185,18 @@ class Table(object):
         if indexes is not None:
             table.indexes = indexes
 
+        if global_indexes is not None:
+            table.global_indexes = global_indexes
+
         # Prep the schema.
         raw_schema = []
         attr_defs = []
+        seen_attrs = set()
 
         for field in table.schema:
             raw_schema.append(field.schema())
             # Build the attributes off what we know.
+            seen_attrs.add(field.name)
             attr_defs.append(field.definition())
 
         raw_throughput = {
@@ -170,23 +205,24 @@ class Table(object):
         }
         kwargs = {}
 
-        if table.indexes:
-            # Prep the LSIs.
-            raw_lsi = []
+        kwarg_map = {
+            'indexes': 'local_secondary_indexes',
+            'global_indexes': 'global_secondary_indexes',
+        }
+        for index_attr in ('indexes', 'global_indexes'):
+            table_indexes = getattr(table, index_attr)
+            if table_indexes:
+                raw_indexes = []
+                for index_field in table_indexes:
+                    raw_indexes.append(index_field.schema())
+                    # Make sure all attributes specified in the indexes are
+                    # added to the definition
+                    for field in index_field.parts:
+                        if field.name not in seen_attrs:
+                            seen_attrs.add(field.name)
+                            attr_defs.append(field.definition())
 
-            for index_field in table.indexes:
-                raw_lsi.append(index_field.schema())
-                # Again, build the attributes off what we know.
-                # HOWEVER, only add attributes *NOT* already seen.
-                attr_define = index_field.definition()
-
-                for part in attr_define:
-                    attr_names = [attr['AttributeName'] for attr in attr_defs]
-
-                    if not part['AttributeName'] in attr_names:
-                        attr_defs.append(part)
-
-            kwargs['local_secondary_indexes'] = raw_lsi
+                kwargs[kwarg_map[index_attr]] = raw_indexes
 
         table.connection.create_table(
             table_name=table.table_name,
@@ -197,18 +233,29 @@ class Table(object):
         )
         return table
 
-    def _introspect_schema(self, raw_schema):
+    def _introspect_schema(self, raw_schema, raw_attributes=None):
         """
         Given a raw schema structure back from a DynamoDB response, parse
         out & build the high-level Python objects that represent them.
         """
         schema = []
+        sane_attributes = {}
+
+        if raw_attributes:
+            for field in raw_attributes:
+                sane_attributes[field['AttributeName']] = field['AttributeType']
 
         for field in raw_schema:
+            data_type = sane_attributes.get(field['AttributeName'], STRING)
+
             if field['KeyType'] == 'HASH':
-                schema.append(HashKey(field['AttributeName']))
+                schema.append(
+                    HashKey(field['AttributeName'], data_type=data_type)
+                )
             elif field['KeyType'] == 'RANGE':
-                schema.append(RangeKey(field['AttributeName']))
+                schema.append(
+                    RangeKey(field['AttributeName'], data_type=data_type)
+                )
             else:
                 raise exceptions.UnknownSchemaFieldError(
                     "%s was seen, but is unknown. Please report this at "
@@ -245,7 +292,7 @@ class Table(object):
                 )
 
             name = field['IndexName']
-            kwargs['parts'] = self._introspect_schema(field['KeySchema'])
+            kwargs['parts'] = self._introspect_schema(field['KeySchema'], None)
             indexes.append(index_klass(name, **kwargs))
 
         return indexes
@@ -284,7 +331,8 @@ class Table(object):
         if not self.schema:
             # Since we have the data, build the schema.
             raw_schema = result['Table'].get('KeySchema', [])
-            self.schema = self._introspect_schema(raw_schema)
+            raw_attributes = result['Table'].get('AttributeDefinitions', [])
+            self.schema = self._introspect_schema(raw_schema, raw_attributes)
 
         if not self.indexes:
             # Build the index information as well.
@@ -294,7 +342,7 @@ class Table(object):
         # This is leaky.
         return result
 
-    def update(self, throughput):
+    def update(self, throughput, global_indexes=None):
         """
         Updates table attributes in DynamoDB.
 
@@ -316,12 +364,46 @@ class Table(object):
             ... })
             True
 
+            # To also update the global index(es) throughput.
+            >>> users.update(throughput={
+            ...     'read': 20,
+            ...     'write': 10,
+            ... },
+            ... global_secondary_indexes={
+            ...     'TheIndexNameHere': {
+            ...         'read': 15,
+            ...         'write': 5,
+            ...     }
+            ... })
+            True
+
         """
         self.throughput = throughput
-        self.connection.update_table(self.table_name, {
+        data = {
             'ReadCapacityUnits': int(self.throughput['read']),
             'WriteCapacityUnits': int(self.throughput['write']),
-        })
+        }
+        gsi_data = None
+
+        if global_indexes:
+            gsi_data = []
+
+            for gsi_name, gsi_throughput in global_indexes.items():
+                gsi_data.append({
+                    "Update": {
+                        "IndexName": gsi_name,
+                        "ProvisionedThroughput": {
+                            "ReadCapacityUnits": int(gsi_throughput['read']),
+                            "WriteCapacityUnits": int(gsi_throughput['write']),
+                        },
+                    },
+                })
+
+        self.connection.update_table(
+            self.table_name,
+            provisioned_throughput=data,
+            global_secondary_index_updates=gsi_data
+        )
         return True
 
     def delete(self):
@@ -368,7 +450,7 @@ class Table(object):
 
         return raw_key
 
-    def get_item(self, consistent=False, **kwargs):
+    def get_item(self, consistent=False, attributes=None, **kwargs):
         """
         Fetches an item (record) from a table in DynamoDB.
 
@@ -380,6 +462,10 @@ class Table(object):
         a consistent (but more expensive) read from DynamoDB.
         (Default: ``False``)
 
+        Optionally accepts an ``attributes`` parameter, which should be a
+        list of fieldname to fetch. (Default: ``None``, which means all fields
+        should be fetched)
+
         Returns an ``Item`` instance containing all the data for that record.
 
         Example::
@@ -412,12 +498,54 @@ class Table(object):
         item_data = self.connection.get_item(
             self.table_name,
             raw_key,
+            attributes_to_get=attributes,
             consistent_read=consistent
         )
+        if 'Item' not in item_data:
+            raise exceptions.ItemNotFound("Item %s couldn't be found." % kwargs)
         item = Item(self)
         item.load(item_data)
         return item
 
+    def has_item(self, **kwargs):
+        """
+        Return whether an item (record) exists within a table in DynamoDB.
+
+        To specify the key of the item you'd like to get, you can specify the
+        key attributes as kwargs.
+
+        Optionally accepts a ``consistent`` parameter, which should be a
+        boolean. If you provide ``True``, it will perform
+        a consistent (but more expensive) read from DynamoDB.
+        (Default: ``False``)
+
+        Optionally accepts an ``attributes`` parameter, which should be a
+        list of fieldnames to fetch. (Default: ``None``, which means all fields
+        should be fetched)
+
+        Returns ``True`` if an ``Item`` is present, ``False`` if not.
+
+        Example::
+
+            # Simple, just hash-key schema.
+            >>> users.has_item(username='johndoe')
+            True
+
+            # Complex schema, item not present.
+            >>> users.has_item(
+            ...     username='johndoe',
+            ...     date_joined='2014-01-07'
+            ... )
+            False
+
+        """
+        try:
+            self.get_item(**kwargs)
+        except (JSONResponseError, exceptions.ItemNotFound):
+            return False
+
+        return True
+
     def lookup(self, *args, **kwargs):
         """
         Look up an entry in DynamoDB. This is mostly backwards compatible
@@ -456,7 +584,6 @@ class Table(object):
             data[self.schema[x].name] = arg
         return Item(self, data=data)
 
-
     def put_item(self, data, overwrite=False):
         """
         Saves an entire item to DynamoDB.
@@ -630,6 +757,9 @@ class Table(object):
         An internal method for taking query/scan-style ``**kwargs`` & turning
         them into the raw structure DynamoDB expects for filtering.
         """
+        if filter_kwargs is None:
+            return
+
         filters = {}
 
         for field_and_op, value in filter_kwargs.items():
@@ -687,19 +817,36 @@ class Table(object):
         return filters
 
     def query(self, limit=None, index=None, reverse=False, consistent=False,
-              attributes=None, **filter_kwargs):
+              attributes=None, max_page_size=None, **filter_kwargs):
+        """
+        **WARNING:** This method is provided **strictly** for
+        backward-compatibility. It returns results in an incorrect order.
+
+        If you are writing new code, please use ``Table.query_2``.
+        """
+        reverse = not reverse
+        return self.query_2(limit=limit, index=index, reverse=reverse,
+                            consistent=consistent, attributes=attributes,
+                            max_page_size=max_page_size, **filter_kwargs)
+
+    def query_2(self, limit=None, index=None, reverse=False,
+                consistent=False, attributes=None, max_page_size=None,
+                query_filter=None, conditional_operator=None,
+                **filter_kwargs):
         """
         Queries for a set of matching items in a DynamoDB table.
 
         Queries can be performed against a hash key, a hash+range key or
-        against any data stored in your local secondary indexes.
+        against any data stored in your local secondary indexes. Query filters
+        can be used to filter on arbitrary fields.
 
         **Note** - You can not query against arbitrary fields within the data
-        stored in DynamoDB.
+        stored in DynamoDB unless you specify ``query_filter`` values.
 
         To specify the filters of the items you'd like to get, you can specify
         the filters as kwargs. Each filter kwarg should follow the pattern
-        ``<fieldname>__<filter_operation>=<value_to_look_for>``.
+        ``<fieldname>__<filter_operation>=<value_to_look_for>``. Query filters
+        are specified in the same way.
 
         Optionally accepts a ``limit`` parameter, which should be an integer
         count of the total number of items to return. (Default: ``None`` -
@@ -710,7 +857,7 @@ class Table(object):
         (Default: ``None``)
 
         Optionally accepts a ``reverse`` parameter, which will present the
-        results in reverse order. (Default: ``None`` - normal order)
+        results in reverse order. (Default: ``False`` - normal order)
 
         Optionally accepts a ``consistent`` parameter, which should be a
         boolean. If you provide ``True``, it will force a consistent read of
@@ -722,6 +869,21 @@ class Table(object):
         from DynamoDB. This uses the ``AttributesToGet`` and set's
         ``Select`` to ``SPECIFIC_ATTRIBUTES`` API.
 
+        Optionally accepts a ``max_page_size`` parameter, which should be an
+        integer count of the maximum number of items to retrieve
+        **per-request**. This is useful in making faster requests & prevent
+        the scan from drowning out other queries. (Default: ``None`` -
+        fetch as many as DynamoDB will return)
+
+        Optionally accepts a ``query_filter`` which is a dictionary of filter
+        conditions against any arbitrary field in the returned data.
+
+        Optionally accepts a ``conditional_operator`` which applies to the
+        query filter conditions:
+
+        + `AND` - True if all filter conditions evaluate to true (default)
+        + `OR` - True if at least one filter condition evaluates to true
+
         Returns a ``ResultSet``, which transparently handles the pagination of
         results you get back.
 
@@ -760,19 +922,38 @@ class Table(object):
             'John'
             'Fred'
 
+            # Filter by non-indexed field(s)
+            >>> results = users.query(
+            ...     last_name__eq='Doe',
+            ...     reverse=True,
+            ...     query_filter={
+            ...         'first_name__beginswith': 'A'
+            ...     }
+            ... )
+            >>> for res in results:
+            ...     print res['first_name'] + ' ' + res['last_name']
+            'Alice Doe'
+
         """
         if self.schema:
-            if len(self.schema) == 1 and len(filter_kwargs) <= 1:
-                raise exceptions.QueryError(
-                    "You must specify more than one key to filter on."
-                )
+            if len(self.schema) == 1:
+                if len(filter_kwargs) <= 1:
+                    if not self.global_indexes or not len(self.global_indexes):
+                        # If the schema only has one field, there's <= 1 filter
+                        # param & no Global Secondary Indexes, this is user
+                        # error. Bail early.
+                        raise exceptions.QueryError(
+                            "You must specify more than one key to filter on."
+                        )
 
         if attributes is not None:
             select = 'SPECIFIC_ATTRIBUTES'
         else:
             select = None
 
-        results = ResultSet()
+        results = ResultSet(
+            max_page_size=max_page_size
+        )
         kwargs = filter_kwargs.copy()
         kwargs.update({
             'limit': limit,
@@ -780,21 +961,26 @@ class Table(object):
             'reverse': reverse,
             'consistent': consistent,
             'select': select,
-            'attributes_to_get': attributes
+            'attributes_to_get': attributes,
+            'query_filter': query_filter,
+            'conditional_operator': conditional_operator,
         })
         results.to_call(self._query, **kwargs)
         return results
 
-    def query_count(self, index=None, consistent=False, **filter_kwargs):
+    def query_count(self, index=None, consistent=False, conditional_operator=None,
+                    query_filter=None, **filter_kwargs):
         """
         Queries the exact count of matching items in a DynamoDB table.
 
         Queries can be performed against a hash key, a hash+range key or
-        against any data stored in your local secondary indexes.
+        against any data stored in your local secondary indexes. Query filters
+        can be used to filter on arbitrary fields.
 
         To specify the filters of the items you'd like to get, you can specify
         the filters as kwargs. Each filter kwarg should follow the pattern
-        ``<fieldname>__<filter_operation>=<value_to_look_for>``.
+        ``<fieldname>__<filter_operation>=<value_to_look_for>``. Query filters
+        are specified in the same way.
 
         Optionally accepts an ``index`` parameter, which should be a string of
         name of the local secondary index you want to query against.
@@ -805,6 +991,15 @@ class Table(object):
         the data (more expensive). (Default: ``False`` - use eventually
         consistent reads)
 
+        Optionally accepts a ``query_filter`` which is a dictionary of filter
+        conditions against any arbitrary field in the returned data.
+
+        Optionally accepts a ``conditional_operator`` which applies to the
+        query filter conditions:
+
+        + `AND` - True if all filter conditions evaluate to true (default)
+        + `OR` - True if at least one filter condition evaluates to true
+
         Returns an integer which represents the exact amount of matched
         items.
 
@@ -829,18 +1024,25 @@ class Table(object):
             using=QUERY_OPERATORS
         )
 
+        built_query_filter = self._build_filters(
+            query_filter,
+            using=FILTER_OPERATORS
+        )
+
         raw_results = self.connection.query(
             self.table_name,
             index_name=index,
             consistent_read=consistent,
             select='COUNT',
             key_conditions=key_conditions,
+            query_filter=built_query_filter,
+            conditional_operator=conditional_operator,
         )
         return int(raw_results.get('Count', 0))
 
     def _query(self, limit=None, index=None, reverse=False, consistent=False,
                exclusive_start_key=None, select=None, attributes_to_get=None,
-               **filter_kwargs):
+               query_filter=None, conditional_operator=None, **filter_kwargs):
         """
         The internal method that performs the actual queries. Used extensively
         by ``ResultSet`` to perform each (paginated) request.
@@ -848,12 +1050,15 @@ class Table(object):
         kwargs = {
             'limit': limit,
             'index_name': index,
-            'scan_index_forward': reverse,
             'consistent_read': consistent,
             'select': select,
-            'attributes_to_get': attributes_to_get
+            'attributes_to_get': attributes_to_get,
+            'conditional_operator': conditional_operator,
         }
 
+        if reverse:
+            kwargs['scan_index_forward'] = False
+
         if exclusive_start_key:
             kwargs['exclusive_start_key'] = {}
 
@@ -867,6 +1072,11 @@ class Table(object):
             using=QUERY_OPERATORS
         )
 
+        kwargs['query_filter'] = self._build_filters(
+            query_filter,
+            using=FILTER_OPERATORS
+        )
+
         raw_results = self.connection.query(
             self.table_name,
             **kwargs
@@ -893,13 +1103,14 @@ class Table(object):
         }
 
     def scan(self, limit=None, segment=None, total_segments=None,
+             max_page_size=None, attributes=None, conditional_operator=None,
              **filter_kwargs):
         """
         Scans across all items within a DynamoDB table.
 
         Scans can be performed against a hash key or a hash+range key. You can
         additionally filter the results after the table has been read but
-        before the response is returned.
+        before the response is returned by using query filters.
 
         To specify the filters of the items you'd like to get, you can specify
         the filters as kwargs. Each filter kwarg should follow the pattern
@@ -909,6 +1120,26 @@ class Table(object):
         count of the total number of items to return. (Default: ``None`` -
         all results)
 
+        Optionally accepts a ``segment`` parameter, which should be an integer
+        of the segment to retrieve on. Please see the documentation about
+        Parallel Scans (Default: ``None`` - no segments)
+
+        Optionally accepts a ``total_segments`` parameter, which should be an
+        integer count of number of segments to divide the table into.
+        Please see the documentation about Parallel Scans (Default: ``None`` -
+        no segments)
+
+        Optionally accepts a ``max_page_size`` parameter, which should be an
+        integer count of the maximum number of items to retrieve
+        **per-request**. This is useful in making faster requests & prevent
+        the scan from drowning out other queries. (Default: ``None`` -
+        fetch as many as DynamoDB will return)
+
+        Optionally accepts an ``attributes`` parameter, which should be a
+        tuple. If you provide any attributes only these will be fetched
+        from DynamoDB. This uses the ``AttributesToGet`` and set's
+        ``Select`` to ``SPECIFIC_ATTRIBUTES`` API.
+
         Returns a ``ResultSet``, which transparently handles the pagination of
         results you get back.
 
@@ -935,18 +1166,23 @@ class Table(object):
             'Alice'
 
         """
-        results = ResultSet()
+        results = ResultSet(
+            max_page_size=max_page_size
+        )
         kwargs = filter_kwargs.copy()
         kwargs.update({
             'limit': limit,
             'segment': segment,
             'total_segments': total_segments,
+            'attributes': attributes,
+            'conditional_operator': conditional_operator,
         })
         results.to_call(self._scan, **kwargs)
         return results
 
     def _scan(self, limit=None, exclusive_start_key=None, segment=None,
-              total_segments=None, **filter_kwargs):
+              total_segments=None, attributes=None, conditional_operator=None,
+              **filter_kwargs):
         """
         The internal method that performs the actual scan. Used extensively
         by ``ResultSet`` to perform each (paginated) request.
@@ -955,6 +1191,8 @@ class Table(object):
             'limit': limit,
             'segment': segment,
             'total_segments': total_segments,
+            'attributes_to_get': attributes,
+            'conditional_operator': conditional_operator,
         }
 
         if exclusive_start_key:
@@ -1032,7 +1270,7 @@ class Table(object):
         # We pass the keys to the constructor instead, so it can maintain it's
         # own internal state as to what keys have been processed.
         results = BatchGetResultSet(keys=keys, max_batch_get=self.max_batch_get)
-        results.to_call(self._batch_get, consistent=False)
+        results.to_call(self._batch_get, consistent=consistent)
         return results
 
     def _batch_get(self, keys, consistent=False):
