@@ -2,19 +2,21 @@ import mock
 import unittest
 from boto.dynamodb2 import exceptions
 from boto.dynamodb2.fields import (HashKey, RangeKey,
-                                   AllIndex, KeysOnlyIndex, IncludeIndex)
+                                   AllIndex, KeysOnlyIndex, IncludeIndex,
+                                   GlobalAllIndex, GlobalKeysOnlyIndex,
+                                   GlobalIncludeIndex)
 from boto.dynamodb2.items import Item
 from boto.dynamodb2.layer1 import DynamoDBConnection
 from boto.dynamodb2.results import ResultSet, BatchGetResultSet
 from boto.dynamodb2.table import Table
-from boto.dynamodb2.types import (STRING, NUMBER,
+from boto.dynamodb2.types import (STRING, NUMBER, BINARY,
                                   FILTER_OPERATORS, QUERY_OPERATORS)
+from boto.exception import JSONResponseError
 
 
 FakeDynamoDBConnection = mock.create_autospec(DynamoDBConnection)
 
 
-
 class SchemaFieldsTestCase(unittest.TestCase):
     def test_hash_key(self):
         hash_key = HashKey('hello')
@@ -169,6 +171,170 @@ class IndexFieldTestCase(unittest.TestCase):
             }
         })
 
+    def test_global_all_index(self):
+        all_index = GlobalAllIndex('AllKeys', parts=[
+            HashKey('username'),
+            RangeKey('date_joined')
+        ],
+        throughput={
+            'read': 6,
+            'write': 2,
+        })
+        self.assertEqual(all_index.name, 'AllKeys')
+        self.assertEqual([part.attr_type for part in all_index.parts], [
+            'HASH',
+            'RANGE'
+        ])
+        self.assertEqual(all_index.projection_type, 'ALL')
+
+        self.assertEqual(all_index.definition(), [
+            {'AttributeName': 'username', 'AttributeType': 'S'},
+            {'AttributeName': 'date_joined', 'AttributeType': 'S'}
+        ])
+        self.assertEqual(all_index.schema(), {
+            'IndexName': 'AllKeys',
+            'KeySchema': [
+                {
+                    'AttributeName': 'username',
+                    'KeyType': 'HASH'
+                },
+                {
+                    'AttributeName': 'date_joined',
+                    'KeyType': 'RANGE'
+                }
+            ],
+            'Projection': {
+                'ProjectionType': 'ALL'
+            },
+            'ProvisionedThroughput': {
+                'ReadCapacityUnits': 6,
+                'WriteCapacityUnits': 2
+            }
+        })
+
+    def test_global_keys_only_index(self):
+        keys_only = GlobalKeysOnlyIndex('KeysOnly', parts=[
+            HashKey('username'),
+            RangeKey('date_joined')
+        ],
+        throughput={
+            'read': 3,
+            'write': 4,
+        })
+        self.assertEqual(keys_only.name, 'KeysOnly')
+        self.assertEqual([part.attr_type for part in keys_only.parts], [
+            'HASH',
+            'RANGE'
+        ])
+        self.assertEqual(keys_only.projection_type, 'KEYS_ONLY')
+
+        self.assertEqual(keys_only.definition(), [
+            {'AttributeName': 'username', 'AttributeType': 'S'},
+            {'AttributeName': 'date_joined', 'AttributeType': 'S'}
+        ])
+        self.assertEqual(keys_only.schema(), {
+            'IndexName': 'KeysOnly',
+            'KeySchema': [
+                {
+                    'AttributeName': 'username',
+                    'KeyType': 'HASH'
+                },
+                {
+                    'AttributeName': 'date_joined',
+                    'KeyType': 'RANGE'
+                }
+            ],
+            'Projection': {
+                'ProjectionType': 'KEYS_ONLY'
+            },
+            'ProvisionedThroughput': {
+                'ReadCapacityUnits': 3,
+                'WriteCapacityUnits': 4
+            }
+        })
+
+    def test_global_include_index(self):
+        # Lean on the default throughput
+        include_index = GlobalIncludeIndex('IncludeKeys', parts=[
+            HashKey('username'),
+            RangeKey('date_joined')
+        ], includes=[
+            'gender',
+            'friend_count'
+        ])
+        self.assertEqual(include_index.name, 'IncludeKeys')
+        self.assertEqual([part.attr_type for part in include_index.parts], [
+            'HASH',
+            'RANGE'
+        ])
+        self.assertEqual(include_index.projection_type, 'INCLUDE')
+
+        self.assertEqual(include_index.definition(), [
+            {'AttributeName': 'username', 'AttributeType': 'S'},
+            {'AttributeName': 'date_joined', 'AttributeType': 'S'}
+        ])
+        self.assertEqual(include_index.schema(), {
+            'IndexName': 'IncludeKeys',
+            'KeySchema': [
+                {
+                    'AttributeName': 'username',
+                    'KeyType': 'HASH'
+                },
+                {
+                    'AttributeName': 'date_joined',
+                    'KeyType': 'RANGE'
+                }
+            ],
+            'Projection': {
+                'ProjectionType': 'INCLUDE',
+                'NonKeyAttributes': [
+                    'gender',
+                    'friend_count',
+                ]
+            },
+            'ProvisionedThroughput': {
+                'ReadCapacityUnits': 5,
+                'WriteCapacityUnits': 5
+            }
+        })
+
+    def test_global_include_index_throughput(self):
+        include_index = GlobalIncludeIndex('IncludeKeys', parts=[
+            HashKey('username'),
+            RangeKey('date_joined')
+        ], includes=[
+            'gender',
+            'friend_count'
+        ], throughput={
+            'read': 10,
+            'write': 8
+        })
+
+        self.assertEqual(include_index.schema(), {
+            'IndexName': 'IncludeKeys',
+            'KeySchema': [
+                {
+                    'AttributeName': 'username',
+                    'KeyType': 'HASH'
+                },
+                {
+                    'AttributeName': 'date_joined',
+                    'KeyType': 'RANGE'
+                }
+            ],
+            'Projection': {
+                'ProjectionType': 'INCLUDE',
+                'NonKeyAttributes': [
+                    'gender',
+                    'friend_count',
+                ]
+            },
+            'ProvisionedThroughput': {
+                'ReadCapacityUnits': 10,
+                'WriteCapacityUnits': 8
+            }
+        })
+
 
 class ItemTestCase(unittest.TestCase):
     def setUp(self):
@@ -476,7 +642,7 @@ class ItemTestCase(unittest.TestCase):
             'date_joined'
         ]))
 
-    def test_prepare_partial(self):
+    def test_prepare_partial_empty_set(self):
         self.johndoe.mark_clean()
         # Change some data.
         self.johndoe['first_name'] = 'Johann'
@@ -691,11 +857,33 @@ class ResultSetTestCase(unittest.TestCase):
     def setUp(self):
         super(ResultSetTestCase, self).setUp()
         self.results = ResultSet()
-        self.results.to_call(fake_results, 'john', greeting='Hello', limit=20)
+        self.result_function = mock.MagicMock(side_effect=fake_results)
+        self.results.to_call(self.result_function, 'john', greeting='Hello', limit=20)
 
     def test_first_key(self):
         self.assertEqual(self.results.first_key, 'exclusive_start_key')
 
+    def test_max_page_size_fetch_more(self):
+        self.results = ResultSet(max_page_size=10)
+        self.results.to_call(self.result_function, 'john', greeting='Hello')
+        self.results.fetch_more()
+        self.result_function.assert_called_with('john', greeting='Hello', limit=10)
+        self.result_function.reset_mock()
+
+    def test_max_page_size_and_smaller_limit_fetch_more(self):
+        self.results = ResultSet(max_page_size=10)
+        self.results.to_call(self.result_function, 'john', greeting='Hello', limit=5)
+        self.results.fetch_more()
+        self.result_function.assert_called_with('john', greeting='Hello', limit=5)
+        self.result_function.reset_mock()
+
+    def test_max_page_size_and_bigger_limit_fetch_more(self):
+        self.results = ResultSet(max_page_size=10)
+        self.results.to_call(self.result_function, 'john', greeting='Hello', limit=15)
+        self.results.fetch_more()
+        self.result_function.assert_called_with('john', greeting='Hello', limit=10)
+        self.result_function.reset_mock()
+
     def test_fetch_more(self):
         # First "page".
         self.results.fetch_more()
@@ -707,6 +895,9 @@ class ResultSetTestCase(unittest.TestCase):
             'Hello john #4',
         ])
 
+        self.result_function.assert_called_with('john', greeting='Hello', limit=20)
+        self.result_function.reset_mock()
+
         # Fake in a last key.
         self.results._last_key_seen = 4
         # Second "page".
@@ -719,6 +910,9 @@ class ResultSetTestCase(unittest.TestCase):
             'Hello john #9',
         ])
 
+        self.result_function.assert_called_with('john', greeting='Hello', limit=20, exclusive_start_key=4)
+        self.result_function.reset_mock()
+
         # Fake in a last key.
         self.results._last_key_seen = 9
         # Last "page".
@@ -745,20 +939,20 @@ class ResultSetTestCase(unittest.TestCase):
         self.assertEqual(self.results.next(), 'Hello john #2')
         self.assertEqual(self.results.next(), 'Hello john #3')
         self.assertEqual(self.results.next(), 'Hello john #4')
-        self.assertEqual(self.results.call_kwargs['limit'], 15)
+        self.assertEqual(self.results._limit, 15)
         # Second page.
         self.assertEqual(self.results.next(), 'Hello john #5')
         self.assertEqual(self.results.next(), 'Hello john #6')
         self.assertEqual(self.results.next(), 'Hello john #7')
         self.assertEqual(self.results.next(), 'Hello john #8')
         self.assertEqual(self.results.next(), 'Hello john #9')
-        self.assertEqual(self.results.call_kwargs['limit'], 10)
+        self.assertEqual(self.results._limit, 10)
         # Third page.
         self.assertEqual(self.results.next(), 'Hello john #10')
         self.assertEqual(self.results.next(), 'Hello john #11')
         self.assertEqual(self.results.next(), 'Hello john #12')
         self.assertRaises(StopIteration, self.results.next)
-        self.assertEqual(self.results.call_kwargs['limit'], 7)
+        self.assertEqual(self.results._limit, 7)
 
     def test_limit_smaller_than_first_page(self):
         results = ResultSet()
@@ -960,7 +1154,17 @@ class TableTestCase(unittest.TestCase):
                 "KeyType": "RANGE"
             }
         ]
-        schema_1 = self.users._introspect_schema(raw_schema_1)
+        raw_attributes_1 = [
+            {
+                'AttributeName': 'username',
+                'AttributeType': 'S'
+            },
+            {
+                'AttributeName': 'date_joined',
+                'AttributeType': 'S'
+            },
+        ]
+        schema_1 = self.users._introspect_schema(raw_schema_1, raw_attributes_1)
         self.assertEqual(len(schema_1), 2)
         self.assertTrue(isinstance(schema_1[0], HashKey))
         self.assertEqual(schema_1[0].name, 'username')
@@ -973,12 +1177,49 @@ class TableTestCase(unittest.TestCase):
                 "KeyType": "BTREE"
             },
         ]
+        raw_attributes_2 = [
+            {
+                'AttributeName': 'username',
+                'AttributeType': 'S'
+            },
+        ]
         self.assertRaises(
             exceptions.UnknownSchemaFieldError,
             self.users._introspect_schema,
-            raw_schema_2
+            raw_schema_2,
+            raw_attributes_2
         )
 
+        # Test a complex schema & ensure the types come back correctly.
+        raw_schema_3 = [
+            {
+                "AttributeName": "user_id",
+                "KeyType": "HASH"
+            },
+            {
+                "AttributeName": "junk",
+                "KeyType": "RANGE"
+            }
+        ]
+        raw_attributes_3 = [
+            {
+                'AttributeName': 'user_id',
+                'AttributeType': 'N'
+            },
+            {
+                'AttributeName': 'junk',
+                'AttributeType': 'B'
+            },
+        ]
+        schema_3 = self.users._introspect_schema(raw_schema_3, raw_attributes_3)
+        self.assertEqual(len(schema_3), 2)
+        self.assertTrue(isinstance(schema_3[0], HashKey))
+        self.assertEqual(schema_3[0].name, 'user_id')
+        self.assertEqual(schema_3[0].data_type, NUMBER)
+        self.assertTrue(isinstance(schema_3[1], RangeKey))
+        self.assertEqual(schema_3[1].name, 'junk')
+        self.assertEqual(schema_3[1].data_type, BINARY)
+
     def test__introspect_indexes(self):
         raw_indexes_1 = [
             {
@@ -1132,6 +1373,13 @@ class TableTestCase(unittest.TestCase):
                 KeysOnlyIndex('FriendCountIndex', parts=[
                     RangeKey('friend_count')
                 ]),
+            ], global_indexes=[
+                GlobalKeysOnlyIndex('FullFriendCountIndex', parts=[
+                    RangeKey('friend_count')
+                ], throughput={
+                    'read': 10,
+                    'write': 8,
+                }),
             ], connection=conn)
             self.assertTrue(retval)
 
@@ -1165,6 +1413,24 @@ class TableTestCase(unittest.TestCase):
             'WriteCapacityUnits': 10,
             'ReadCapacityUnits': 20
         },
+        global_secondary_indexes=[
+            {
+                'KeySchema': [
+                    {
+                        'KeyType': 'RANGE',
+                        'AttributeName': 'friend_count'
+                    }
+                ],
+                'IndexName': 'FullFriendCountIndex',
+                'Projection': {
+                    'ProjectionType': 'KEYS_ONLY'
+                },
+                'ProvisionedThroughput': {
+                    'WriteCapacityUnits': 8,
+                    'ReadCapacityUnits': 10
+                }
+            }
+        ],
         local_secondary_indexes=[
             {
                 'KeySchema': [
@@ -1252,10 +1518,65 @@ class TableTestCase(unittest.TestCase):
             self.assertEqual(self.users.throughput['read'], 7)
             self.assertEqual(self.users.throughput['write'], 2)
 
-        mock_update.assert_called_once_with('users', {
-            'WriteCapacityUnits': 2,
-            'ReadCapacityUnits': 7
-        })
+        mock_update.assert_called_once_with(
+            'users',
+            global_secondary_index_updates=None,
+            provisioned_throughput={
+                'WriteCapacityUnits': 2,
+                'ReadCapacityUnits': 7
+            }
+        )
+
+        with mock.patch.object(
+                self.users.connection,
+                'update_table',
+                return_value={}) as mock_update:
+            self.assertEqual(self.users.throughput['read'], 7)
+            self.assertEqual(self.users.throughput['write'], 2)
+            self.users.update(throughput={
+                'read': 9,
+                'write': 5,
+            },
+            global_indexes={
+                'WhateverIndex': {
+                    'read': 6,
+                    'write': 1
+                },
+                'AnotherIndex': {
+                    'read': 1,
+                    'write': 2
+                }
+            })
+            self.assertEqual(self.users.throughput['read'], 9)
+            self.assertEqual(self.users.throughput['write'], 5)
+
+        mock_update.assert_called_once_with(
+            'users',
+            global_secondary_index_updates=[
+                {
+                    'Update': {
+                        'IndexName': 'AnotherIndex',
+                        'ProvisionedThroughput': {
+                            'WriteCapacityUnits': 2,
+                            'ReadCapacityUnits': 1
+                        }
+                    }
+                },
+                {
+                    'Update': {
+                        'IndexName': 'WhateverIndex',
+                        'ProvisionedThroughput': {
+                            'WriteCapacityUnits': 1,
+                            'ReadCapacityUnits': 6
+                        }
+                    }
+                }
+            ],
+            provisioned_throughput={
+                'WriteCapacityUnits': 5,
+                'ReadCapacityUnits': 9,
+            }
+        )
 
     def test_delete(self):
         with mock.patch.object(
@@ -1288,7 +1609,46 @@ class TableTestCase(unittest.TestCase):
 
         mock_get_item.assert_called_once_with('users', {
             'username': {'S': 'johndoe'}
-        }, consistent_read=False)
+        }, consistent_read=False, attributes_to_get=None)
+
+        with mock.patch.object(
+                self.users.connection,
+                'get_item',
+                return_value=expected) as mock_get_item:
+            item = self.users.get_item(username='johndoe', attributes=[
+                'username',
+                'first_name',
+            ])
+
+        mock_get_item.assert_called_once_with('users', {
+            'username': {'S': 'johndoe'}
+        }, consistent_read=False, attributes_to_get=['username', 'first_name'])
+
+    def test_has_item(self):
+        expected = {
+            'Item': {
+                'username': {'S': 'johndoe'},
+                'first_name': {'S': 'John'},
+                'last_name': {'S': 'Doe'},
+                'date_joined': {'N': '1366056668'},
+                'friend_count': {'N': '3'},
+                'friends': {'SS': ['alice', 'bob', 'jane']},
+            }
+        }
+
+        with mock.patch.object(
+                self.users.connection,
+                'get_item',
+                return_value=expected) as mock_get_item:
+            found = self.users.has_item(username='johndoe')
+            self.assertTrue(found)
+
+        with mock.patch.object(
+                self.users.connection,
+                'get_item') as mock_get_item:
+            mock_get_item.side_effect = JSONResponseError("Nope.", None, None)
+            found = self.users.has_item(username='mrsmith')
+            self.assertFalse(found)
 
     def test_lookup_hash(self):
         """Tests the "lookup" function with just a hash key"""
@@ -1347,7 +1707,6 @@ class TableTestCase(unittest.TestCase):
             username= 'johndoe',
             date_joined= 1366056668)
 
-
     def test_put_item(self):
         with mock.patch.object(
                 self.users.connection,
@@ -1837,7 +2196,7 @@ class TableTestCase(unittest.TestCase):
 
         mock_query.assert_called_once_with('users',
             consistent_read=False,
-            scan_index_forward=True,
+            scan_index_forward=False,
             index_name=None,
             attributes_to_get=None,
             limit=4,
@@ -1847,7 +2206,9 @@ class TableTestCase(unittest.TestCase):
                     'ComparisonOperator': 'BETWEEN',
                 }
             },
-            select=None
+            select=None,
+            query_filter=None,
+            conditional_operator=None
         )
 
         # Now alter the expected.
@@ -1868,7 +2229,9 @@ class TableTestCase(unittest.TestCase):
                 exclusive_start_key={
                     'username': 'adam',
                 },
-                consistent=True
+                consistent=True,
+                query_filter=None,
+                conditional_operator='AND'
             )
             usernames = [res['username'] for res in results['results']]
             self.assertEqual(usernames, ['johndoe', 'jane', 'alice', 'bob'])
@@ -1884,7 +2247,7 @@ class TableTestCase(unittest.TestCase):
             },
             index_name=None,
             attributes_to_get=None,
-            scan_index_forward=True,
+            scan_index_forward=False,
             limit=4,
             exclusive_start_key={
                 'username': {
@@ -1892,7 +2255,9 @@ class TableTestCase(unittest.TestCase):
                 },
             },
             consistent_read=True,
-            select=None
+            select=None,
+            query_filter=None,
+            conditional_operator='AND'
         )
 
     def test_private_scan(self):
@@ -1953,7 +2318,9 @@ class TableTestCase(unittest.TestCase):
             },
             limit=2,
             segment=None,
-            total_segments=None
+            attributes_to_get=None,
+            total_segments=None,
+            conditional_operator=None
         )
 
         # Now alter the expected.
@@ -1995,7 +2362,9 @@ class TableTestCase(unittest.TestCase):
                 },
             },
             segment=None,
-            total_segments=None
+            attributes_to_get=None,
+            total_segments=None,
+            conditional_operator=None
         )
 
     def test_query(self):
@@ -2015,7 +2384,7 @@ class TableTestCase(unittest.TestCase):
             'last_key': 'jane',
         }
 
-        results = self.users.query(last_name__eq='Doe')
+        results = self.users.query_2(last_name__eq='Doe')
         self.assertTrue(isinstance(results, ResultSet))
         self.assertEqual(len(results._results), 0)
         self.assertEqual(results.the_callable, self.users._query)
@@ -2069,7 +2438,7 @@ class TableTestCase(unittest.TestCase):
             'last_key': 'jane',
         }
 
-        results = self.users.query(last_name__eq='Doe',
+        results = self.users.query_2(last_name__eq='Doe',
                                    attributes=['username'])
         self.assertTrue(isinstance(results, ResultSet))
         self.assertEqual(len(results._results), 0)
@@ -2147,6 +2516,38 @@ class TableTestCase(unittest.TestCase):
 
         self.assertEqual(mock_scan_2.call_count, 1)
 
+    def test_scan_with_specific_attributes(self):
+        items_1 = {
+            'results': [
+                Item(self.users, data={
+                    'username': 'johndoe',
+                }),
+                Item(self.users, data={
+                    'username': 'jane',
+                }),
+            ],
+            'last_key': 'jane',
+        }
+
+        results = self.users.scan(attributes=['username'])
+        self.assertTrue(isinstance(results, ResultSet))
+        self.assertEqual(len(results._results), 0)
+        self.assertEqual(results.the_callable, self.users._scan)
+
+        with mock.patch.object(
+                results,
+                'the_callable',
+                return_value=items_1) as mock_query:
+            res_1 = results.next()
+            # Now it should be populated.
+            self.assertEqual(len(results._results), 2)
+            self.assertEqual(res_1['username'], 'johndoe')
+            self.assertEqual(res_1.keys(), ['username'])
+            res_2 = results.next()
+            self.assertEqual(res_2['username'], 'jane')
+
+        self.assertEqual(mock_query.call_count, 1)
+
     def test_count(self):
         expected = {
             "Table": {
